package jalview.ws.ebi;

import jalview.datamodel.AlignmentAnnotation;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.SequenceI;
import jalview.io.FileParse;

import java.io.IOException;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

public class HmmerJSONProcessor
{
  AlignmentI resultAl;

  public HmmerJSONProcessor(AlignmentI searchResult)
  {
    resultAl = searchResult;
  }

  public void parseFrom(FileParse jsonsource) throws IOException,
          OutOfMemoryError
  {
    JSONParser hmmerResultParser = new JSONParser();
    Object jsonResults = null;
    try
    {
      jsonResults = hmmerResultParser.parse(jsonsource.getReader());
    } catch (Exception p)
    {
      throw new IOException("While parsing from " + jsonsource.getInFile(),
              p);
    }
    if (jsonResults == null)
    {
      throw new IOException("No data at" + jsonsource.getInFile());
    }
    if (!(jsonResults instanceof JSONObject))
    {
      throw new IOException("Unexpected JSON model at "
              + jsonsource.getInFile());
    }
    try
    {
      JSONObject hmmsearchr = (JSONObject) ((JSONObject) jsonResults)
              .get("results");
      // now process the hits
      addStatistics((JSONObject) hmmsearchr.get("stats"));
      JSONArray jsonArray = (JSONArray) hmmsearchr.get("hits");
      long p = 1;
      for (Object hit : jsonArray)
      {
        JSONObject hmmhit = (JSONObject) hit;
        addHit(hmmhit, p++);
      }
    } catch (ClassCastException q)
    {
      throw new IOException("Unexpected JSON model content at "
              + jsonsource.getInFile(), q);
    }
  }

  /**
   * 
   * @param object
   *          - actually a JSONObject key value set of search statistics.
   */
  public void addStatistics(JSONObject stats)
  {
    for (Object stat : stats.keySet())
    {
      String key = (String) stat;
      Object val = stats.get(key);
      resultAl.setProperty(key, "" + val);
    }
  }

  // encodings for JSON keys
  /**
   * score becomes sequence associated AlignmentAnnotation
   */
  private String[] score = { "aliId", "ali_IdCount", "bitscore", "ievalue",
      "aliSim", "aliSimCount", "aliL", "aliSim", "ievalue", "cevalue" };

  /**
   * attrib becomes numeric or binary attribute for sequence with respect to
   * this hmmsearch run
   */
  private String[] attrib = { "bias", "oasc", "is_included", "is_reported" };

  /**
   * name of the hmmsearch query
   */
  private String[] label = { "alihmmname" // (query label?)},
  };

  /**
   * integer attributes for each
   */
  private String[] ipos = { "alihmmfrom", "alihmmto" }, pos_l = {
      "alimline", "alimodel", "alirfline" };

  /**
   * positional quantitative annotation encoded as strings.
   */
  private String[] pos_nscore = { "alippline" };

  //
  // mapping of keys to types of property on sequence
  //
  public void addHit(JSONObject hmmrhit, long p)
  {
    String sname = (String) hmmrhit.get("name");
    SequenceI[] hits = resultAl.findSequenceMatch(sname);
    if (hits == null)
    {
      System.err.println("No seq for " + sname);
    }
    double pvalue = (Double) hmmrhit.get("pvalue");

    double evalue = Double.valueOf("" + hmmrhit.get("evalue"));
    for (Object domainhit : ((JSONArray) hmmrhit.get("domains")))
    {
      JSONObject dhit=(JSONObject) domainhit;
      // dhit.get(key)

      // alihmmfrom,alihmmto alimodel
      // alisqfrom,alisqto,aliaseq
      // alippline
      // 
      for (SequenceI hitseq : hits)
      {
        // match alisqfrom,alisqto,seq
        // overlay ppline as seqannotation
        // ievalue
        // cevalue
        // 

        AlignmentAnnotation pval = new AlignmentAnnotation("p-value",
                "hmmer3 pvalue", pvalue);
        AlignmentAnnotation eval = new AlignmentAnnotation("e-value",
                "hmmer3 evalue", evalue);
        hitseq.addAlignmentAnnotation(pval);
        hitseq.addAlignmentAnnotation(eval);
        

      }
    }
  }
}
